/*
 *  SingIt Lyrics Displayer
 *  Copyright (C) 2000 - 2003 Jan-Marek Glogowski <glogow@stud.fbi.fh-darmstadt.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "macros.h"

#include <glib.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include "singit_config.h"
#include "singit_creator.h"
#include "colorbutton.h"

#include <xmms/configfile.h>

extern SingitConfig *singit_config;

SingitConfig* singit_config_new(void)
{
	SingitConfig* result;

#ifdef DEBUG
	puts("singit_config.c [singit_config_new]");
#endif

	result = g_malloc(sizeof(SingitConfig));

	if (result != NULL) {
		result->isLoaded = FALSE;
		result->basePath = NULL;
		result->lyricExtension = NULL;
		result->lyricFontName = NULL;

		result->backgroundColor = NULL;
		result->normalTextColor = NULL;
		result->normalBarColor = NULL;
		result->progressTextColor = NULL;
		result->progressBarColor = NULL;

		result->attachments = 1;
	}

	return result;
}

void singit_config_free(SingitConfig* config)
{
	if (!config) { return; }

#ifdef DEBUG
	puts("singit_config.c [singit_config_free]");
#endif

	g_free(config->basePath);
	g_free(config->lyricExtension);
	g_free(config->lyricFontName);

	g_free(config->backgroundColor);
	g_free(config->normalTextColor);
	g_free(config->normalBarColor);
	g_free(config->progressTextColor);
	g_free(config->progressBarColor);

	g_free(config);
}

void singit_config_load(SingitConfig* config)
{
	ConfigFile *cfg;
	gchar *filename = NULL;
/*	gchar *text = NULL, *test;
	GString *string = NULL;*/

#ifdef DEBUG
	printf("singit_config.c [singit_config_load] : ");
#endif

	if (config == NULL) { config = singit_config_new(); }

	g_free(config->basePath);
	g_free(config->lyricExtension);
	g_free(config->lyricFontName);

	g_free(config->backgroundColor);
	g_free(config->normalTextColor);
	g_free(config->normalBarColor);
	g_free(config->progressTextColor);
	g_free(config->progressBarColor);

	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	cfg = xmms_cfg_open_file(filename);
	if (!cfg) { cfg = xmms_cfg_new(); }

	if (!xmms_cfg_read_boolean(cfg, "SingIt", "lyricPolling", &config->lyricPolling))
		{ config->lyricPolling = TRUE; }
	if (!xmms_cfg_read_boolean(cfg, "SingIt", "toggleYZ", &config->toggleYZ))
		{ config->toggleYZ = TRUE; }
	if (!xmms_cfg_read_boolean(cfg, "SingIt", "showEmpty", &config->showEmpty))
		{ config->showEmpty = FALSE; }
	if (!xmms_cfg_read_boolean(cfg, "SingIt", "rolling", &config->rolling))
		{ config->rolling = FALSE; }
	if (!xmms_cfg_read_boolean(cfg, "SingIt", "autoResize", &config->autoResize))
		{ config->autoResize = TRUE; }
	if (!xmms_cfg_read_boolean(cfg, "SingIt", "hideIfNotFound", &config->hideIfNotFound))
		{ config->hideIfNotFound = FALSE; }

	if (!xmms_cfg_read_string(cfg, "SingIt", "basePath", &(config->basePath)))
		{ config->basePath = g_strdup("~/lyrics/"); }
	if (!xmms_cfg_read_string(cfg, "SingIt", "lyricExtension", &(config->lyricExtension)))
		{ config->lyricExtension = g_strdup("txt"); }
	if (!xmms_cfg_read_string(cfg, "SingIt", "lyricFontName", &(config->lyricFontName)))
		{ config->lyricFontName = g_strdup("-adobe-helvetica-bold-r-*-*-14-*"); }

	if (!(xmms_cfg_read_string(cfg, "SingIt", "backgroundColor", &config->backgroundColor)))
		{ config->backgroundColor = g_strdup("white"); }
	if (!(xmms_cfg_read_string(cfg, "SingIt", "normalTextColor", &config->normalTextColor)))
		{ config->normalTextColor = g_strdup("black"); }
	if (!(xmms_cfg_read_string(cfg, "SingIt", "normalBarColor", &config->normalBarColor)))
		{ config->normalBarColor = g_strdup("white"); }
	if (!(xmms_cfg_read_string(cfg, "SingIt", "progressTextColor", &config->progressTextColor)))
		{ config->progressTextColor = g_strdup("black"); }
	if (!(xmms_cfg_read_string(cfg, "SingIt", "progressBarColor", &config->progressBarColor)))
		{ config->progressBarColor = g_strdup("lightblue"); }

	if (!(xmms_cfg_read_int(cfg, "SingIt", "singitWinPosX", &config->singitWinPosX)))
		{ config->singitWinPosX = -1; }
	if (!(xmms_cfg_read_int(cfg, "SingIt", "singitWinPosY", &config->singitWinPosY)))
		{ config->singitWinPosY = -1; }
	if (!(xmms_cfg_read_int(cfg, "SingIt", "creatorWinPosX", &config->creatorWinPosX)))
		{ config->creatorWinPosX = -1; }
	if (!(xmms_cfg_read_int(cfg, "SingIt", "creatorWinPosY", &config->creatorWinPosY)))
		{ config->creatorWinPosY = -1; }
	if (!(xmms_cfg_read_int(cfg, "SingIt", "configWinPosX", &config->configWinPosX)))
		{ config->configWinPosX = -1; }
	if (!(xmms_cfg_read_int(cfg, "SingIt", "configWinPosY", &config->configWinPosY)))
		{ config->configWinPosY = -1; }
	if (!(xmms_cfg_read_int(cfg, "SingIt", "singitWinSizeY", &config->singitWinSizeY)))
		{ config->singitWinSizeY = 250; }

// * From lyrics editor *
	if (!xmms_cfg_read_boolean(cfg, "SingIt", "multiSelect", &config->multiSelect))
		{ config->multiSelect = FALSE; }
	if (!xmms_cfg_read_boolean(cfg, "SingIt", "showHints", &config->showHints))
		{ config->showHints = TRUE; }
	if (!xmms_cfg_read_boolean(cfg, "SingIt", "addSameTime", &config->addSameTime))
		{ config->addSameTime = FALSE; }

	xmms_cfg_free(cfg);
	g_free(filename);

	config->isLoaded = TRUE;
#ifdef DEBUG
	puts("Ok");
#endif
}

void singit_config_editor_save(SingitConfig *config)
{
	ConfigFile *cfg;
	gchar *filename;

#ifdef DEBUG
	printf("singit_config.c [singit_config_editor_save] : ");
#endif

	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	cfg = xmms_cfg_open_file(filename);
	if (!cfg)
		cfg = xmms_cfg_new();

	xmms_cfg_write_boolean(cfg, "SingIt", "multiSelect", config->multiSelect);
	xmms_cfg_write_boolean(cfg, "SingIt", "showHints", config->showHints);
	xmms_cfg_write_boolean(cfg, "SingIt", "addSameTime", config->addSameTime);

	xmms_cfg_write_file(cfg, filename);
	xmms_cfg_free(cfg);
	g_free(filename);

#ifdef DEBUG
	puts("Ok");
#endif
}

gchar* create_string(gchar *text, gboolean free)
{
	gchar **lines, *result;

	lines = g_strsplit (text, "\n", 1000);
	result = g_strjoinv ("\\n", lines);
	g_strfreev(lines);
	if (free) { g_free(text); }
	return result;
}

void xmms_cfg_write_del_string(ConfigFile *cfg, gchar *section, gchar* entry, gchar *value)
{
	if (value == NULL) { xmms_cfg_remove_key(cfg, section, entry); }
	else { xmms_cfg_write_string(cfg, section, entry, value); }
}

void singit_config_save(SingitConfig* config)
{
	ConfigFile *cfg;
	gchar *filename;

	if (!config) { return; }

#ifdef DEBUG
	printf("singit_config.c [singit_config_save] : ");
#endif

	filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	cfg = xmms_cfg_open_file(filename);
	if (!cfg)
		cfg = xmms_cfg_new();

	xmms_cfg_write_boolean(cfg, "SingIt", "lyricPolling", config->lyricPolling);
	xmms_cfg_write_boolean(cfg, "SingIt", "toggleYZ", config->toggleYZ);
	xmms_cfg_write_boolean(cfg, "SingIt", "showEmpty", config->showEmpty);
	xmms_cfg_write_boolean(cfg, "SingIt", "rolling", config->rolling);
	xmms_cfg_write_boolean(cfg, "SingIt", "autoResize", config->autoResize);
	xmms_cfg_write_boolean(cfg, "SingIt", "hideIfNotFound", config->hideIfNotFound);

	xmms_cfg_write_del_string(cfg, "SingIt", "basePath", config->basePath);
	xmms_cfg_write_del_string(cfg, "SingIt", "lyricExtension", config->lyricExtension);
	xmms_cfg_write_del_string(cfg, "SingIt", "lyricFontName", config->lyricFontName);

	xmms_cfg_write_del_string(cfg, "SingIt", "backgroundColor", config->backgroundColor);
	xmms_cfg_write_del_string(cfg, "SingIt", "normalTextColor", config->normalTextColor);
	xmms_cfg_write_del_string(cfg, "SingIt", "normalBarColor", config->normalBarColor);
	xmms_cfg_write_del_string(cfg, "SingIt", "progressTextColor", config->progressTextColor);
	xmms_cfg_write_del_string(cfg, "SingIt", "progressBarColor", config->progressBarColor);

	xmms_cfg_write_int(cfg, "SingIt", "singitWinPosX", config->singitWinPosX);
	xmms_cfg_write_int(cfg, "SingIt", "singitWinPosY", config->singitWinPosY);
	xmms_cfg_write_int(cfg, "SingIt", "creatorWinPosX", config->creatorWinPosX);
	xmms_cfg_write_int(cfg, "SingIt", "creatorWinPosY", config->creatorWinPosY);
	xmms_cfg_write_int(cfg, "SingIt", "configWinPosX", config->configWinPosX);
	xmms_cfg_write_int(cfg, "SingIt", "configWinPosY", config->configWinPosY);
	xmms_cfg_write_int(cfg, "SingIt", "singitWinSizeY", config->singitWinSizeY);
	xmms_cfg_write_file(cfg, filename);
	xmms_cfg_free(cfg);
	g_free(filename);

#ifdef DEBUG
	puts("Ok");
#endif
}

gboolean singit_config_attach(SingitConfig *config)
{
#ifdef DEBUG
	printf("singit_config.c [singit_config_attach] : ");
#endif

	if (config != NULL) {
		config->attachments++;
#ifdef DEBUG
		puts("Attached");
#endif
		return TRUE;
	}
#ifdef DEBUG
	puts("Failed");
#endif
	return FALSE;
}

gboolean singit_config_detach(SingitConfig *config, gboolean free)
{
#ifdef DEBUG
	printf("singit_config.c [singit_config_detach] : ");
#endif

	if (!config) { return TRUE; }
	config->attachments--;
	if ((config->attachments == 0) && (free)) {
#ifdef DEBUG
		puts("Freed");
#endif
		singit_config_free(config);
		return TRUE;
	}
#ifdef DEBUG
	puts("Detached");
#endif
	return FALSE;
}
