/*
 *  SingIt Lyrics Displayer
 *  Copyright (C) 2000 - 2003 Jan-Marek Glogowski <glogow@stud.fbi.fh-darmstadt.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "macros.h"

#include <glib.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include "singit_config.h"
#include "singit_main.h"
#include "singit_creator.h"
#include "colorbutton.h"

static GtkWidget *singit_config_win = NULL;
static GtkWidget
	*config_vbox,
		*config_notebook,
		*config_displayer_vbox, *config_displayer_label,
			*config_filefound_frame,
				*config_filefound_vbox,
				*config_filefound_basepath_label,
				*config_filefound_basepath_entry,
				*config_filefound_hsep1,
				*config_filefound_extension_label,
				*config_filefound_extension_entry,
			*config_font_frame,
				*config_font_hbox,
				*config_font_entry,
				*config_font_browse,
				*config_fontsel,
			*config_toggle_yz_check_button,
			*config_show_empty_check_button,
			*config_auto_resize_check_button,
			*config_hide_if_not_found_check_button,

		*config_color_vbox, *config_color_label,
			*config_color_frame,
				*config_color_table,
				*config_color_background_label,
				*config_color_background_button,
				*config_color_normal_text_label,
				*config_color_normal_text_button,
				*config_color_normal_bar_label,
				*config_color_normal_bar_button,
				*config_color_progress_text_label,
				*config_color_progress_text_button,
				*config_color_progress_bar_label,
				*config_color_progress_bar_button,

		*config_creator_hbox, *config_creator_label,
			*config_creator_vbox,
				*config_creator_start_button,
				*config_creator_check_button_vbox,
				*config_creator_multi_select_check_button,
				*config_show_hints_check_button,
				*config_add_same_time_check_button,

		*bottom_button_hbox,
			*bottom_button_ok,
			*bottom_button_cancel,
			*bottom_button_apply;

typedef struct {

	guchar *color_array;

} ConfigStatus;

ConfigStatus config_status;
extern SingitConfig *singit_config;

static gboolean singit_config_apply_clicked(GtkButton *button, gpointer user_data)
{
	gchar tmp[13];
	SingitConfig *config = singit_config;

#ifdef DEBUG
	printf("singit_config.c [singit_config_dialog_save] : ");
#endif

	if ((config == NULL) || (!singit_config_win)) {
#ifdef DEBUG
		puts("failed");
#endif

		return FALSE;
	}

	tmp[12] = '\0';

	/* clear old values */
	g_free(config->basePath);
	g_free(config->lyricExtension);
	g_free(config->lyricFontName);

	g_free(config->backgroundColor);
	g_free(config->normalTextColor);
	g_free(config->normalBarColor);
	g_free(config->progressTextColor);
	g_free(config->progressBarColor);

	/* set new values */
	config->basePath = g_strdup(g_strstrip(gtk_entry_get_text(GTK_ENTRY(config_filefound_basepath_entry))));

	config->lyricExtension = g_strdup(g_strstrip(gtk_entry_get_text(GTK_ENTRY(config_filefound_extension_entry))));
	config->lyricFontName = g_strdup(g_strstrip(gtk_entry_get_text(GTK_ENTRY(config_font_entry))));
	config->toggleYZ = GTK_TOGGLE_BUTTON(config_toggle_yz_check_button)->active;
	config->showEmpty = GTK_TOGGLE_BUTTON(config_show_empty_check_button)->active;
	config->autoResize = GTK_TOGGLE_BUTTON(config_auto_resize_check_button)->active;
	config->hideIfNotFound = GTK_TOGGLE_BUTTON(config_hide_if_not_found_check_button)->active;

#define GET_RGB_AS_STRING(a,b) \
	do { \
		sprintf(tmp, "rgb:%.2x/%.2x/%.2x", \
		(gushort) ((guchar *) (COLOR_BUTTON(a)->color))[0], \
		(gushort) ((guchar *) (COLOR_BUTTON(a)->color))[1], \
		(gushort) ((guchar *) (COLOR_BUTTON(a)->color))[2]); \
		b = g_strdup(tmp); \
	} while (0)

	GET_RGB_AS_STRING(config_color_background_button, config->backgroundColor);
	GET_RGB_AS_STRING(config_color_normal_text_button, config->normalTextColor);
	GET_RGB_AS_STRING(config_color_normal_bar_button, config->normalBarColor);
	GET_RGB_AS_STRING(config_color_progress_text_button, config->progressTextColor);
	GET_RGB_AS_STRING(config_color_progress_bar_button, config->progressBarColor);

#undef GET_RGB_AS_STRING

	config->multiSelect = GTK_TOGGLE_BUTTON(config_creator_multi_select_check_button)->active;
	config->showHints = GTK_TOGGLE_BUTTON(config_show_hints_check_button)->active;
	config->addSameTime = GTK_TOGGLE_BUTTON(config_add_same_time_check_button)->active;

#ifdef DEBUG
	puts("Ok");
#endif

	singit_config_save(config);
	singit_config_editor_save(config);

	singit_main_change_style();

	return TRUE;
}

static void config_font_browse_ok(GtkWidget * w, gpointer data)
{
        gtk_entry_set_text(GTK_ENTRY(config_font_entry),
		gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(config_fontsel)));
        gtk_widget_destroy(config_fontsel);
}

static void config_font_browse_cb(GtkWidget * w, gpointer data)
{
        if (!config_fontsel)
        {
                config_fontsel = gtk_font_selection_dialog_new(_("Select lyric font:"));
                gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(config_fontsel),
			singit_config->lyricFontName);
                gtk_signal_connect(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(config_fontsel)->ok_button), "clicked",
			GTK_SIGNAL_FUNC(config_font_browse_ok), NULL);
                gtk_signal_connect_object(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(config_fontsel)->cancel_button),
			"clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(config_fontsel));
                gtk_signal_connect(GTK_OBJECT(config_fontsel), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			&config_fontsel);
                gtk_widget_show(config_fontsel);
        }
}

void start_singit_creator(GtkWidget *button, gpointer config)
{
	singit_config_editor_save(singit_config);
	singit_creator_show();
}

static void singit_config_init()
{
#ifdef DEBUG
	puts("singit_config.c [singit_config_init]");
#endif

	config_status.color_array = g_malloc(sizeof(guchar) * 15);
}

static void singit_config_cleanup()
{
	if (singit_config_win && GTK_WIDGET_REALIZED(singit_config_win)) {
		gdk_window_get_position(singit_config_win->window,
			&(singit_config->configWinPosX),
			&(singit_config->configWinPosY));
	}

	g_free(config_status.color_array);

	singit_config_save(singit_config);
	if (singit_config_detach(singit_config, TRUE))
		singit_config = NULL;
}

static void singit_config_ok_clicked(GtkButton *button, gpointer user_data)
{
	singit_config_apply_clicked(NULL, NULL);
	singit_config_cleanup();
	gtk_widget_destroy(singit_config_win);
}

static void singit_config_cancel_clicked(GtkButton *button, gpointer user_data)
{
	singit_config_cleanup();
	gtk_widget_destroy(singit_config_win);
}

static gint singit_config_dlg_delete_event( GtkWidget *widget, GdkEvent  *event, gpointer data )
{
	singit_config_cleanup();
	return (FALSE);
}

void singit_config_hide(void)
{
	if (!singit_config_win) return;

#ifdef DEBUG
	puts("singit_config.c [singit_config_hide]");
#endif

	gtk_widget_destroy(singit_config_win);
}

static void add_color_button(gchar *trans_label, gchar *color_str, gint col_arr_offset,
	gint row, GtkWidget *table, GtkWidget **col_label, GtkWidget **col_button)
{
	GdkColor color;

	gdk_color_parse(color_str, &color);
	config_status.color_array[col_arr_offset] = color.red;
	config_status.color_array[col_arr_offset + 1] = color.green;
	config_status.color_array[col_arr_offset + 2] = color.blue;

	*col_label = gtk_label_new(trans_label);
	gtk_table_attach (GTK_TABLE(table),
		*col_label, 0, 1, row, row + 1,
		GTK_SHRINK | GTK_FILL | GTK_EXPAND, GTK_SHRINK | GTK_FILL, 0, 0);
	gtk_misc_set_alignment(GTK_MISC(*col_label), 1, 0.5);
	*col_button = color_button_new(trans_label, 60, 20,
		&config_status.color_array[col_arr_offset], 3);
	gtk_table_attach (GTK_TABLE(table),
		*col_button, 1, 2, row, row + 1,
		GTK_SHRINK | GTK_EXPAND, GTK_SHRINK | GTK_FILL, 0, 0);

	gtk_widget_show(*col_label);
	gtk_widget_show(*col_button);
}

static void append_color_page()
{
	config_color_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(config_color_vbox), 2);
	gtk_widget_show (config_color_vbox);
	config_color_label = gtk_label_new(_("Colors"));
	gtk_widget_show(config_color_label);
	gtk_notebook_append_page(GTK_NOTEBOOK(config_notebook),
		config_color_vbox, config_color_label);

// * color selection frame *
	config_color_frame = gtk_frame_new(_("Colors"));
	gtk_box_pack_start(GTK_BOX(config_color_vbox),
		config_color_frame, FALSE, FALSE, 0);
	gtk_widget_show(config_color_frame);

	config_color_table = gtk_table_new (2, 5, FALSE);
	gtk_table_set_row_spacings (GTK_TABLE (config_color_table), 3);
	gtk_table_set_col_spacings (GTK_TABLE (config_color_table), 5);
	gtk_container_set_border_width(GTK_CONTAINER(config_color_table), 3);
	gtk_container_add(GTK_CONTAINER(config_color_frame), config_color_table);
	gtk_widget_show(config_color_table);

	add_color_button(_("Background Color"), singit_config->backgroundColor, 0, 0,
		config_color_table,
		&config_color_background_label,
		&config_color_background_button);

	add_color_button(_("Normal Text Color"), singit_config->normalTextColor, 3, 1,
		config_color_table,
		&config_color_normal_text_label,
		&config_color_normal_text_button);

	add_color_button(_("Normal Bar Color"), singit_config->normalBarColor, 6, 2,
		config_color_table,
		&config_color_normal_bar_label,
		&config_color_normal_bar_button);

	add_color_button(_("Progress Text Color"), singit_config->progressTextColor, 9, 3,
		config_color_table,
		&config_color_progress_text_label,
		&config_color_progress_text_button);

	add_color_button(_("Progress Bar Color"), singit_config->progressBarColor, 12, 4,
		config_color_table,
		&config_color_progress_bar_label,
		&config_color_progress_bar_button);
}

void singit_config_show(void)
{
#ifdef DEBUG
	printf("singit_config.c [singit_config_show] : ");
#endif

	if (!singit_config_attach(singit_config)) {
		singit_config = singit_config_new();
		singit_config_load(singit_config);
	}

	g_return_if_fail(singit_config != NULL);

	if (singit_config_win) {
#ifdef DEBUG
		puts("failed - config_win exists");
#endif
		return;
	}

#ifdef DEBUG
	puts("Ok");
#endif

#ifdef ENABLE_NLS
	setlocale (LC_ALL, "");
	bindtextdomain (PACKAGE, LOCALE_DIR);
#endif

	singit_config_init();

	singit_config_win = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_signal_connect(GTK_OBJECT(singit_config_win), "destroy",
		GTK_SIGNAL_FUNC(gtk_widget_destroyed), &singit_config_win);
	gtk_signal_connect(GTK_OBJECT(singit_config_win), "delete_event",
		GTK_SIGNAL_FUNC(singit_config_dlg_delete_event), NULL);
	gtk_window_set_title(GTK_WINDOW(singit_config_win), SINGIT_VERSION_STRING " Config");
	gtk_window_set_policy(GTK_WINDOW(singit_config_win), FALSE, FALSE, FALSE);
	gtk_container_border_width(GTK_CONTAINER(singit_config_win), 5);

	config_vbox = gtk_vbox_new(FALSE, 5);
	gtk_container_add(GTK_CONTAINER(singit_config_win), config_vbox);
	gtk_widget_show(config_vbox);

	config_notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(config_notebook), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(config_vbox), config_notebook, FALSE, FALSE, 0);
	gtk_widget_show(config_notebook);

// * lyrics displyer config page *
	config_displayer_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(config_displayer_vbox), 2);
	gtk_widget_show (config_displayer_vbox);
	config_displayer_label = gtk_label_new(_("Lyrics displayer"));
	gtk_widget_show(config_displayer_label);
	gtk_notebook_append_page(GTK_NOTEBOOK(config_notebook), config_displayer_vbox, config_displayer_label);

	config_filefound_frame = gtk_frame_new(_("Lyric file constructor"));
	gtk_box_pack_start(GTK_BOX(config_displayer_vbox), config_filefound_frame, FALSE, FALSE, 0);
	gtk_widget_show(config_filefound_frame);

	config_filefound_vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(config_filefound_vbox), 5);
	gtk_container_add(GTK_CONTAINER(config_filefound_frame), config_filefound_vbox);
	gtk_widget_show(config_filefound_vbox);

// * lyric basepath *
	config_filefound_basepath_label = gtk_label_new(_("Lyrics dir"));
	gtk_box_pack_start(GTK_BOX(config_filefound_vbox), config_filefound_basepath_label, TRUE, TRUE, 0);
	gtk_widget_show(config_filefound_basepath_label);

	config_filefound_basepath_entry = gtk_entry_new();
        gtk_box_pack_start(GTK_BOX(config_filefound_vbox), config_filefound_basepath_entry, TRUE, TRUE, 0);
	gtk_entry_set_text(GTK_ENTRY(config_filefound_basepath_entry), singit_config->basePath);
	gtk_widget_show(config_filefound_basepath_entry);

	config_filefound_hsep1 = gtk_hseparator_new();
        gtk_box_pack_start(GTK_BOX(config_filefound_vbox), config_filefound_hsep1, FALSE, TRUE, 0);
	gtk_widget_show(config_filefound_hsep1);

// * lyric extension *
	config_filefound_extension_label = gtk_label_new(_("Lyric extension"));
	gtk_box_pack_start(GTK_BOX(config_filefound_vbox), config_filefound_extension_label, TRUE, TRUE, 0);
	gtk_widget_show(config_filefound_extension_label);

        config_filefound_extension_entry = gtk_entry_new();
        gtk_box_pack_start(GTK_BOX(config_filefound_vbox), config_filefound_extension_entry, TRUE, TRUE, 0);
	gtk_entry_set_text(GTK_ENTRY(config_filefound_extension_entry), singit_config->lyricExtension);
	gtk_widget_show(config_filefound_extension_entry);

// * font selection box (from XMMS) *
        config_font_frame = gtk_frame_new(_("Lyrics font"));
        gtk_box_pack_start(GTK_BOX(config_displayer_vbox), config_font_frame, FALSE, FALSE, 0);
	gtk_widget_show(config_font_frame);

        config_font_hbox = gtk_hbox_new(FALSE, 5);
        gtk_container_border_width(GTK_CONTAINER(config_font_hbox), 5);
        gtk_container_add(GTK_CONTAINER(config_font_frame), config_font_hbox);
	gtk_widget_show(config_font_hbox);

        config_font_entry = gtk_entry_new();
        gtk_box_pack_start(GTK_BOX(config_font_hbox), config_font_entry, TRUE, TRUE, 0);
	gtk_entry_set_text(GTK_ENTRY(config_font_entry), singit_config->lyricFontName);
	gtk_widget_show(config_font_entry);

        config_font_browse = gtk_button_new_with_label("...");
        gtk_signal_connect(GTK_OBJECT(config_font_browse), "clicked", GTK_SIGNAL_FUNC(config_font_browse_cb), NULL);
        gtk_box_pack_start(GTK_BOX(config_font_hbox), config_font_browse, FALSE, FALSE, 0);
	gtk_widget_show(config_font_browse);

// * lyrics displayer checkboxes *
	config_toggle_yz_check_button = gtk_check_button_new_with_label(_("Toggle y/z-key"));
	gtk_box_pack_start(GTK_BOX(config_displayer_vbox), config_toggle_yz_check_button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_toggle_yz_check_button), singit_config->toggleYZ);
	gtk_widget_show(config_toggle_yz_check_button);

	config_show_empty_check_button = gtk_check_button_new_with_label(_("Show empty lines"));
	gtk_box_pack_start(GTK_BOX(config_displayer_vbox), config_show_empty_check_button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_show_empty_check_button), singit_config->showEmpty);
	gtk_widget_show(config_show_empty_check_button);

	config_auto_resize_check_button = gtk_check_button_new_with_label(_("Automagically resize lyric displayer"));
	gtk_box_pack_start(GTK_BOX(config_displayer_vbox), config_auto_resize_check_button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_auto_resize_check_button), singit_config->autoResize);
	gtk_widget_show(config_auto_resize_check_button);

	config_hide_if_not_found_check_button = gtk_check_button_new_with_label(_("Hide displayer if no lyrics were found"));
	gtk_box_pack_start(GTK_BOX(config_displayer_vbox), config_hide_if_not_found_check_button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_hide_if_not_found_check_button), singit_config->hideIfNotFound);
	gtk_widget_show(config_hide_if_not_found_check_button);

	append_color_page();

// * lyrics editor config page *
	config_creator_hbox = gtk_hbox_new(FALSE, 1);
	gtk_container_set_border_width(GTK_CONTAINER(config_creator_hbox), 2);
	gtk_widget_show(config_creator_hbox);

	config_creator_label = gtk_label_new(_("Lyrics creator"));
	gtk_notebook_append_page(GTK_NOTEBOOK(config_notebook), config_creator_hbox, config_creator_label);
	gtk_widget_show(config_creator_label);

// * lyrics creator config box *
	config_creator_vbox = gtk_vbox_new(TRUE, 1);
	gtk_box_pack_start(GTK_BOX(config_creator_hbox), config_creator_vbox, TRUE, FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(config_creator_vbox), 2);
	gtk_widget_show(config_creator_vbox);

	config_creator_start_button = gtk_button_new_with_label(_("Start lyrics creator ..."));
	gtk_signal_connect(GTK_OBJECT(config_creator_start_button), "clicked", GTK_SIGNAL_FUNC(start_singit_creator), NULL);
	gtk_box_pack_start(GTK_BOX(config_creator_vbox), config_creator_start_button, TRUE, FALSE, 0);
	gtk_widget_show(config_creator_start_button);

	config_creator_check_button_vbox = gtk_vbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(config_creator_vbox), config_creator_check_button_vbox, TRUE, FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(config_creator_check_button_vbox), 2);
	gtk_widget_show(config_creator_check_button_vbox);

	config_creator_multi_select_check_button = gtk_check_button_new_with_label(_("Allow multiple lyric selections"));
	gtk_box_pack_start(GTK_BOX(config_creator_check_button_vbox), config_creator_multi_select_check_button, TRUE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_creator_multi_select_check_button), singit_config->multiSelect);
	gtk_widget_show(config_creator_multi_select_check_button);

	config_show_hints_check_button = gtk_check_button_new_with_label(_("Show creators toolbar tooltips"));
	gtk_box_pack_start(GTK_BOX(config_creator_check_button_vbox), config_show_hints_check_button, TRUE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_show_hints_check_button), singit_config->showHints);
	gtk_widget_show(config_show_hints_check_button);

	config_add_same_time_check_button = gtk_check_button_new_with_label(_("Allow multiple same timestamps"));
	gtk_box_pack_start(GTK_BOX(config_creator_check_button_vbox), config_add_same_time_check_button, TRUE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_add_same_time_check_button), singit_config->addSameTime);
	gtk_widget_show(config_add_same_time_check_button);

//* Bottom button row ([Ok] [Cancel][Apply]) *
	bottom_button_hbox = gtk_hbutton_box_new();
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bottom_button_hbox), 5);
	gtk_box_pack_start(GTK_BOX(config_vbox), bottom_button_hbox, FALSE, FALSE, 0);
	gtk_widget_show(bottom_button_hbox);

	bottom_button_ok = gtk_button_new_with_label(_("Ok"));
	gtk_signal_connect_object(GTK_OBJECT(bottom_button_ok),
		"clicked", GTK_SIGNAL_FUNC(singit_config_ok_clicked), GTK_OBJECT(singit_config_win));
	GTK_WIDGET_SET_FLAGS(bottom_button_ok, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(bottom_button_hbox), bottom_button_ok, TRUE, TRUE, 0);
	gtk_widget_show(bottom_button_ok);

	bottom_button_cancel = gtk_button_new_with_label(_("Cancel"));
	gtk_signal_connect_object(GTK_OBJECT(bottom_button_cancel),
		"clicked", GTK_SIGNAL_FUNC(singit_config_cancel_clicked), NULL);
	GTK_WIDGET_SET_FLAGS(bottom_button_cancel, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(bottom_button_hbox), bottom_button_cancel, TRUE, TRUE, 0);
	gtk_widget_grab_default(bottom_button_cancel);
	gtk_widget_show(bottom_button_cancel);

	bottom_button_apply = gtk_button_new_with_label(_("Apply"));
	gtk_signal_connect_object(GTK_OBJECT(bottom_button_apply),
		"clicked", GTK_SIGNAL_FUNC(singit_config_apply_clicked), NULL);
	GTK_WIDGET_SET_FLAGS(bottom_button_apply, GTK_CAN_DEFAULT);
	gtk_box_pack_start(GTK_BOX(bottom_button_hbox), bottom_button_apply, TRUE, TRUE, 0);
	gtk_widget_show(bottom_button_apply);

	gtk_widget_show(singit_config_win);

	if ((singit_config->configWinPosX < 0) || (singit_config->configWinPosY < 0)) {
		gtk_window_set_position
			(GTK_WINDOW(singit_config_win), GTK_WIN_POS_MOUSE);
	}
	else {
		gdk_window_move(singit_config_win->window,
			singit_config->configWinPosX, singit_config->configWinPosY);
	}
}
