/*
 *  SingIt Lyrics Displayer
 *  Copyright (C) 2000 - 2002 Jan-Marek Glogowski <glogow@stud.fbi.fh-darmstadt.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef __SONG_H__
#define __SONG_H__

#include "macros.h"

#include <glib.h>

#ifdef HAVE_ID3
#include <id3.h>
#endif

#include "singit_file_info.h"

typedef struct {

	guint  line;	// Lyric line
	guint  time;	// Play time in milliseconds
	guint pos;	// Used by synchronized lyrics to get the position in the line

} LToken;

typedef struct {

	gint lyrics_found;	// See l_song_lyrics_found

	GList *first_token;
	GList *last_token;
	GList *next_token;
	GList *prev_token;
	guint  lyrics_length;	// It's set, but not used

	gchar **lyrics;		// Lyrics text
	gint    lyric_lines;

	gchar *delimiter;	// It's set, but not used
	gboolean form_Id3v2;

	gchar  *artist;
	gchar  *title;
	gchar  *album;

#ifdef HAVE_ID3
	ID3Tag *id3tag;
#endif

	SingitFileInfo  *fi_song;
	SingitFileInfo  *fi_lyrics;

	gboolean editable;	// Flag to provide information
				// If this song is editable by the creator
} LSong;

#define tTime(a) (((LToken*) (a->data))->time)
#define tLine(a) (((LToken*) (a->data))->line)
#define tPos(a) (((LToken*) (a->data))->pos)
#define l_song_text_found(a) (((LSong *) a)->lyrics != NULL)
#define l_song_lyrics_found(a) (((LSong *) a)->first_token != NULL)

LSong* l_song_new(gchar* song_filename);
void l_song_clear(LSong* song);
void l_song_free(LSong* song);

inline gint inl_l_song_is_time_ok(LSong *song, gint time);
gint l_song_is_time_ok(LSong *song, gint time);

gint l_song_find_line(LSong* song, gint line);
gint l_song_find_time(LSong* song, gint line);
GList* l_song_find_current_token(LSong *song, gint time, gint state);

/* convert a string into a song */
gboolean l_song_build_lyrics(LSong *song, gchar *text);
/* convert a file into a song */
gboolean l_song_load_lyrics(LSong *song, gchar *filename);
/* convert a song into a file : filename = NULL -> use song file */
gboolean l_song_save_lyrics(LSong *song, gchar *filename);
/* returns a string, which can be saved as the lyrics file */
gchar* l_song_create_stream(LSong *song, gint tag_type);

gboolean l_song_lyrics_changed(LSong *song);
gboolean l_song_changed(LSong *song, gchar *filename);

#ifdef HAVE_ID3
void l_song_get_id3_tag(LSong *song, gchar *filename);
#endif

#endif // __SONG_H__
